# -*- coding: utf-8 -*-
# This file is part of the sale-point-invoice module from m-ds.de for
# Tryton.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.

from trytond.tests.test_tryton import with_transaction
from trytond.pool import Pool
from trytond.transaction import Transaction
from trytond.modules.company.tests import create_company, set_company
from trytond.modules.account.tests import create_chart
from decimal import Decimal


class ConfigurationTestCase(object):
    'Test config module'

    def prep_spinv_company_create(self, cname='m-ds'):
        """ create company + currency
        """
        Currency = Pool().get('currency.currency')

        company1 = create_company('m-ds')
        self.assertEqual(company1.party.name, 'm-ds')

        c_lst = Currency.search([])
        self.assertEqual(len(c_lst), 1)
        self.assertEqual(c_lst[0].code, 'usd')

        c_obj, = Currency.create([{
            'name': 'Euro',
            'code': 'EUR',
            'numeric_code': '978',
            'symbol': '€',
            'rounding': Decimal('0.01'),
            'digits': 2}])
        company1.currency = c_obj
        company1.save()
        return company1

    def prep_spinv_add_chart(self, company1):
        """ create chart
        """
        pool = Pool()
        Account = pool.get('account.account')

        with set_company(company1):
            with Transaction().set_context({
                    'company': company1.id, }):
                create_chart(company=company1, tax=True)
                a_lst = Account.search(
                    [('name', '=', 'Main Tax')],
                    order=[('name', 'ASC')])
                self.assertEqual(len(a_lst), 1)

    def prep_spinv_add_product(self, name, account_category):
        """ add product, link category
        """
        pool = Pool()
        Template = pool.get('product.template')
        Product = pool.get('product.product')
        UOM = pool.get('product.uom')

        uom_lst = UOM.search([('symbol', '=', 'u')])
        self.assertEqual(len(uom_lst), 1)
        uom_lst[0].digits = 2
        uom_lst[0].rounding = Decimal('0.01')
        uom_lst[0].save()
        self.assertEqual(uom_lst[0].digits, 2)
        self.assertEqual(uom_lst[0].rounding, 0.01)

        prod1 = Template(
            name=name,
            account_category=account_category,
            gross_price=Decimal('10.0'),
            products=[Product()],
            default_uom=uom_lst[0],
            )
        prod1.on_change_gross_price()
        prod1.save()

        t_lst = Template.search([('name', '=', name)])
        self.assertEqual(len(t_lst), 1)
        self.assertEqual(t_lst[0].name, name)
        self.assertEqual(t_lst[0].gross_price, Decimal('10.0'))
        return t_lst[0]

    def prep_spinv_add_product_category(self, name):
        """ add subject area, check it
        """
        ProdCategory = Pool().get('product.category')

        ProdCategory.create([{'name': name}])
        cat2_lst = ProdCategory.search([('name', '=', name)])
        self.assertEqual(len(cat2_lst), 1)
        self.assertEqual(cat2_lst[0].name, name)
        return cat2_lst[0]

    def prep_spinv_create_prodcategory(self, cat_name):
        """ create category, add tax
        """
        pool = Pool()
        Account = pool.get('account.account')

        a_lst = Account.search([
                ('name', 'in', ['Main Expense', 'Main Revenue'])
            ], order=[('name', 'ASC')])
        self.assertEqual(len(a_lst), 2)
        self.assertEqual(a_lst[0].name, 'Main Expense')
        self.assertEqual(a_lst[1].name, 'Main Revenue')

        p_cat1 = self.prep_spinv_add_product_category(cat_name)
        p_cat1.accounting = True

        p_cat1.account_expense = a_lst[0]
        p_cat1.account_revenue = a_lst[1]
        p_cat1.save()
        return p_cat1

    def prep_spinv_add_tax(self, name, rate):
        """ add tax
        """
        Tax = Pool().get('account.tax')

        tax_templ = Tax.search([('name', '=', '20% VAT')])
        self.assertEqual(len(tax_templ), 1)

        Tax.copy(tax_templ, {
            'name': name,
            'description': name,
            'type': 'percentage',
            'rate': rate,
            })

        tax_lst = Tax.search([('name', '=', name)])
        self.assertEqual(len(tax_lst), 1)
        self.assertEqual(tax_lst[0].name, name)
        self.assertEqual(tax_lst[0].rate, rate)
        self.assertEqual(tax_lst[0].invoice_account.name, 'Main Tax')
        self.assertEqual(tax_lst[0].credit_note_account.name, 'Main Tax')
        return tax_lst[0]

    def prep_spinv_configuration(self):
        """ create config
        """
        pool = Pool()
        Configuration = pool.get('account.configuration')

        company1 = self.prep_spinv_company_create()
        self.prep_spinv_add_chart(company1)

        with set_company(company1):
            with Transaction().set_context({
                    'company': company1.id}):
                cfg1 = Configuration()
                cfg1.save()
        return company1

    @with_transaction()
    def test_spinv_configuration(self):
        """ scan product to empty list
        """
        pool = Pool()
        Configuration = pool.get('account.configuration')

        company1 = self.prep_spinv_configuration()
        with set_company(company1):
            with Transaction().set_context({
                    'company': company1.id}):
                cfg2 = Configuration.get_singleton()
                self.assertEqual(cfg2.modegross, 'net')

# end ConfigurationTestCase
